function p = pdfAiry(zeta,Sk)
% This function returns the probability density function p=p(zeta) for a 
% given vector zeta=eta/sigma and scalar skewness Sk, where eta is the 
% surface elevation and sigma is its standard deviation.  The theoretical 
% function is based on the Airy Ai function, with a semi-empirical left 
% tail added to avoid spurious non-physical oscillations, as derived and 
% described in Fuhrman et al. (2023).  Note that if only the theoretical 
% PDF is desired, without modification of the left tail, simply comment 
% lines 38 and 41 below.
%
% Example usage:
% >> zeta = [-8:0.01:8]; Sk = 0.25; p = pdfAiry(zeta,Sk); % Compute the PDF
% >> semilogy(zeta,p), xlim(8.*[-1 1]), ylim(10.^[-8 0]) % Plot the PDF
% >> xlabel('\zeta=\eta/\sigma'), ylabel('p(\zeta)=p(\eta)\sigma') % Label axes
%
% Programmed by: David R. Fuhrman, Tech. Univ. of Denmark, January 29, 2023
%
% Reference:
%
% Fuhrman, D.R., Klahn, M., Zhai, Y. (2023) A new probability density 
% function for the surface elevation in irregular seas. J. Fluid Mech.
% (submitted).

% Polynomials, for Sk<0.44
if Sk>0.44
    disp('Warning: Skewness is outside the intended range of validity.')
end
alpha = 1 - 73.4946*Sk^4 + 279.701*Sk^5 + 874.876*Sk^6 - 1533.22*Sk^7;
beta = 689.578*Sk^4 - 6545.66*Sk^5 + 16949.1*Sk^6 - 13768.3*Sk^7;
chi0 = -1.17371 + 278.39*Sk^4 - 3068.1*Sk^5 + 9161.14*Sk^6 - 8459.15*Sk^7;

% Create the PDF
argExp = vpa(1/(3*Sk^2) + zeta/Sk); % Argument to exp, high precision
chi = vpa((2/Sk)^(1/3)*(1/(2*Sk)+zeta)); % Argument to Ai, high precision
Zi = airy(chi); % Initialize Zi as the Airy Ai function
i0 = find(chi<chi0); % Indices where chi<chi0
Ci = sqrt(airy(chi(i0)).^2+airy(2,chi(i0)).^2); % Non-oscillatory amplitude
Zi(i0) = alpha*Ci.*(-chi(i0)).^beta; % Modified Ci function;
p = (2/Sk)^(1/3)*exp(argExp).*Zi; % Probability density function (PDF)
area0 = trapz(zeta,p); % Integrate the PDF
p = p/area0; % Re-normalize PDF to ensure unit integral
end